package com.example.jpa;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.ejb.embeddable.EJBContainer;
import javax.naming.Context;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;


public class JPAPerfTest {

	EJBContainer ejbContainer = null;
	
	JPAPerfTestBeanLocal jdbcSB;
	
	@Before
	public void setUp() throws Exception {

		try {
			if (ejbContainer == null)
			{
			    Map<String, Object> properties = new HashMap<String, Object>();
			    File here = new File("./target/test-classes"); 
			    properties.put("org.glassfish.ejb.embedded.glassfish.installation.root", here.getAbsolutePath());
			    ejbContainer = EJBContainer.createEJBContainer(properties);
			    
				Context ctx = ejbContainer.getContext();
	
				jdbcSB = (JPAPerfTestBeanLocal) ctx.lookup("java:global/classes/JPAPerfTestBean");
				
				EntityManagerFactory factory = Persistence.createEntityManagerFactory("testpu");
				EntityManager em = factory.createEntityManager();
				{
					EntityTransaction userTransaction = em.getTransaction();
					try
					{
						userTransaction.begin();
						Query q = em.createNativeQuery("create table customer (id int, name varchar(30))");
						q.executeUpdate();
						userTransaction.commit();
					}
					catch (Exception e)
					{
						userTransaction.rollback();
					}
				
					userTransaction.begin();

					Query q = em.createNativeQuery("select count(*) from customer");
					Integer result = (Integer)q.getSingleResult();
					if (result.intValue() <= 1000)
					{
						for (int i=1; i<=1000; i++)
						{
							q = em.createNativeQuery("insert into customer values(1,'"+i+"')");
							q.executeUpdate();
						}
					}
					userTransaction.commit();
				}				
			}

	    }
	    catch (Exception ex){
			ex.printStackTrace();
			throw new Exception(ex);
	    }
	}
	
	@After
	public void tearDown()
	{
		ejbContainer.close();
	}
	
	@Test
	public void testJDBCPerformance()
	{
		long start = new Date().getTime();
		System.out.println("Start...");
		jdbcSB.doQueryJDBC();
		System.out.println("... finished - took " + (new Date().getTime() - start) + " ms");
	}	
	
	@Test
	public void testJPAPerformance()
	{
		long start = new Date().getTime();
		System.out.println("Start...");
		jdbcSB.doQueryJPA();
		System.out.println("... finished - took " + (new Date().getTime() - start) + " ms");
	}

	
	@Test
	public void testEJBPerformance()
	{
		long start = new Date().getTime();
		System.out.println("Start...");
		jdbcSB.doQueryEJB();
		System.out.println("... finished - took " + (new Date().getTime() - start) + " ms");
	}	
}
